@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -27,16 +29,16 @@ class conv_to
   public:
   
   template<typename in_eT, typename T1>
-  inline static out_eT from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static out_eT from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
 
   template<typename in_eT, typename T1>
-  inline static out_eT from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static out_eT from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static out_eT from(const BaseCube<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static out_eT from(const BaseCube<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static out_eT from(const BaseCube<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static out_eT from(const BaseCube<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   };
 
 
@@ -55,7 +57,7 @@ conv_to<out_eT>::from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_e
   
   const Proxy<T1> P(in.get_ref());
   
-  arma_debug_check( (P.get_n_elem() != 1), "conv_to(): given object doesn't have exactly one element" );
+  arma_debug_check( (P.get_n_elem() != 1), "conv_to(): given object does not have exactly one element" );
   
   return out_eT(Proxy<T1>::use_at ? P.at(0,0) : P[0]);
   }
@@ -76,7 +78,7 @@ conv_to<out_eT>::from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_
   
   const Proxy<T1> P(in.get_ref());
   
-  arma_debug_check( (P.get_n_elem() != 1), "conv_to(): given object doesn't have exactly one element" );
+  arma_debug_check( (P.get_n_elem() != 1), "conv_to(): given object does not have exactly one element" );
   
   out_eT out;
   
@@ -101,7 +103,7 @@ conv_to<out_eT>::from(const BaseCube<in_eT, T1>& in, const typename arma_not_cx<
   
   const ProxyCube<T1> P(in.get_ref());
   
-  arma_debug_check( (P.get_n_elem() != 1), "conv_to(): given object doesn't have exactly one element" );
+  arma_debug_check( (P.get_n_elem() != 1), "conv_to(): given object does not have exactly one element" );
   
   return out_eT(ProxyCube<T1>::use_at ? P.at(0,0,0) : P[0]);
   }
@@ -122,7 +124,7 @@ conv_to<out_eT>::from(const BaseCube<in_eT, T1>& in, const typename arma_cx_only
   
   const ProxyCube<T1> P(in.get_ref());
   
-  arma_debug_check( (P.get_n_elem() != 1), "conv_to(): given object doesn't have exactly one element" );
+  arma_debug_check( (P.get_n_elem() != 1), "conv_to(): given object does not have exactly one element" );
   
   out_eT out;
   
@@ -140,21 +142,26 @@ class conv_to< Mat<out_eT> >
   public:
   
   template<typename in_eT, typename T1>
-  inline static Mat<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Mat<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static Mat<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static Mat<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   
-  template<typename T1>
-  inline static Mat<out_eT> from(const SpBase<out_eT, T1>& in);
+  //
   
+  template<typename in_eT, typename T1>
+  inline static Mat<out_eT> from(const SpBase<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
+  template<typename in_eT, typename T1>
+  inline static Mat<out_eT> from(const SpBase<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
+  
+  //
   
   template<typename in_eT>
-  inline static Mat<out_eT> from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Mat<out_eT> from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT>
-  inline static Mat<out_eT> from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static Mat<out_eT> from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   };
 
 
@@ -172,7 +179,7 @@ conv_to< Mat<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_
   const quasi_unwrap<T1> tmp(in.get_ref());
   const Mat<in_eT>& X  = tmp.M;
   
-  Mat<out_eT> out(X.n_rows, X.n_cols);
+  Mat<out_eT> out(X.n_rows, X.n_cols, arma_nozeros_indicator());
   
   arrayops::convert( out.memptr(), X.memptr(), X.n_elem );
   
@@ -194,7 +201,7 @@ conv_to< Mat<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_o
   const quasi_unwrap<T1> tmp(in.get_ref());
   const Mat<in_eT>& X  = tmp.M;
   
-  Mat<out_eT> out(X.n_rows, X.n_cols);
+  Mat<out_eT> out(X.n_rows, X.n_cols, arma_nozeros_indicator());
   
   arrayops::convert_cx( out.memptr(), X.memptr(), X.n_elem );
   
@@ -204,15 +211,59 @@ conv_to< Mat<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_o
 
 
 template<typename out_eT>
-template<typename T1>
+template<typename in_eT, typename T1>
 arma_warn_unused
 inline
 Mat<out_eT>
-conv_to< Mat<out_eT> >::from(const SpBase<out_eT, T1>& in)
+conv_to< Mat<out_eT> >::from(const SpBase<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
+  arma_ignore(junk);
   
-  return Mat<out_eT>(in.get_ref());
+  const unwrap_spmat<T1> U(in.get_ref());
+  const SpMat<in_eT>&    X = U.M;
+  
+  Mat<out_eT> out(X.n_rows, X.n_cols, arma_zeros_indicator());
+  
+  podarray<out_eT> tmp(X.n_nonzero);
+  
+  arrayops::convert( tmp.memptr(), X.values, X.n_nonzero );
+  
+  typename SpMat<in_eT>::const_iterator it     = X.begin();
+  typename SpMat<in_eT>::const_iterator it_end = X.end();
+  
+  for(uword count=0; it != it_end; ++it, ++count)  { out.at(it.row(), it.col()) = tmp[count]; }
+  
+  return out;
+  }
+
+
+
+template<typename out_eT>
+template<typename in_eT, typename T1>
+arma_warn_unused
+inline
+Mat<out_eT>
+conv_to< Mat<out_eT> >::from(const SpBase<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk)
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  const unwrap_spmat<T1> U(in.get_ref());
+  const SpMat<in_eT>&    X = U.M;
+  
+  Mat<out_eT> out(X.n_rows, X.n_cols, arma_zeros_indicator());
+  
+  podarray<out_eT> tmp(X.n_nonzero);
+  
+  arrayops::convert_cx( tmp.memptr(), X.values, X.n_nonzero );
+  
+  typename SpMat<in_eT>::const_iterator it     = X.begin();
+  typename SpMat<in_eT>::const_iterator it_end = X.end();
+  
+  for(uword count=0; it != it_end; ++it, ++count)  { out.at(it.row(), it.col()) = tmp[count]; }
+  
+  return out;
   }
 
 
@@ -229,7 +280,7 @@ conv_to< Mat<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_n
   
   const uword N = uword( in.size() );
   
-  Mat<out_eT> out(N, 1);
+  Mat<out_eT> out(N, 1, arma_nozeros_indicator());
   
   if(N > 0)
     {
@@ -253,7 +304,7 @@ conv_to< Mat<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_c
   
   const uword N = uword( in.size() );
   
-  Mat<out_eT> out(N, 1);
+  Mat<out_eT> out(N, 1, arma_nozeros_indicator());
   
   if(N > 0)
     {
@@ -272,18 +323,18 @@ class conv_to< Row<out_eT> >
   public:
   
   template<typename in_eT, typename T1>
-  inline static Row<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Row<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static Row<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
-  
+  inline static Row<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   
+  //
   
   template<typename in_eT>
-  inline static Row<out_eT> from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Row<out_eT> from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT>
-  inline static Row<out_eT> from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static Row<out_eT> from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   };
 
 
@@ -301,9 +352,9 @@ conv_to< Row<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_
   const quasi_unwrap<T1> tmp(in.get_ref());
   const Mat<in_eT>& X  = tmp.M;
   
-  arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
+  arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object cannot be interpreted as a vector" );
   
-  Row<out_eT> out(X.n_elem);
+  Row<out_eT> out(X.n_elem, arma_nozeros_indicator());
   
   arrayops::convert( out.memptr(), X.memptr(), X.n_elem );
   
@@ -325,9 +376,9 @@ conv_to< Row<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_o
   const quasi_unwrap<T1> tmp(in.get_ref());
   const Mat<in_eT>& X  = tmp.M;
   
-  arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
+  arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object cannot be interpreted as a vector" );
   
-  Row<out_eT> out(X.n_rows, X.n_cols);
+  Row<out_eT> out(X.n_rows, X.n_cols, arma_nozeros_indicator());
   
   arrayops::convert_cx( out.memptr(), X.memptr(), X.n_elem );
   
@@ -348,7 +399,7 @@ conv_to< Row<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_n
   
   const uword N = uword( in.size() );
   
-  Row<out_eT> out(N);
+  Row<out_eT> out(N, arma_nozeros_indicator());
   
   if(N > 0)
     {
@@ -372,7 +423,7 @@ conv_to< Row<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_c
   
   const uword N = uword( in.size() );
   
-  Row<out_eT> out(N);
+  Row<out_eT> out(N, arma_nozeros_indicator());
   
   if(N > 0)
     {
@@ -391,18 +442,18 @@ class conv_to< Col<out_eT> >
   public:
   
   template<typename in_eT, typename T1>
-  inline static Col<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Col<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static Col<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
-  
+  inline static Col<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   
+  //
   
   template<typename in_eT>
-  inline static Col<out_eT> from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Col<out_eT> from(const std::vector<in_eT>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT>
-  inline static Col<out_eT> from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static Col<out_eT> from(const std::vector<in_eT>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   };
 
 
@@ -420,9 +471,9 @@ conv_to< Col<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_
   const quasi_unwrap<T1> tmp(in.get_ref());
   const Mat<in_eT>& X  = tmp.M;
   
-  arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
+  arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object cannot be interpreted as a vector" );
   
-  Col<out_eT> out(X.n_elem);
+  Col<out_eT> out(X.n_elem, arma_nozeros_indicator());
   
   arrayops::convert( out.memptr(), X.memptr(), X.n_elem );
   
@@ -444,9 +495,9 @@ conv_to< Col<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_o
   const quasi_unwrap<T1> tmp(in.get_ref());
   const Mat<in_eT>& X  = tmp.M;
   
-  arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
+  arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object cannot be interpreted as a vector" );
   
-  Col<out_eT> out(X.n_rows, X.n_cols);
+  Col<out_eT> out(X.n_rows, X.n_cols, arma_nozeros_indicator());
   
   arrayops::convert_cx( out.memptr(), X.memptr(), X.n_elem );
   
@@ -467,7 +518,7 @@ conv_to< Col<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_n
   
   const uword N = uword( in.size() );
   
-  Col<out_eT> out(N);
+  Col<out_eT> out(N, arma_nozeros_indicator());
   
   if(N > 0)
     {
@@ -491,7 +542,7 @@ conv_to< Col<out_eT> >::from(const std::vector<in_eT>& in, const typename arma_c
   
   const uword N = uword( in.size() );
   
-  Col<out_eT> out(N);
+  Col<out_eT> out(N, arma_nozeros_indicator());
   
   if(N > 0)
     {
@@ -510,13 +561,18 @@ class conv_to< SpMat<out_eT> >
   public:
   
   template<typename in_eT, typename T1>
-  inline static SpMat<out_eT> from(const SpBase<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static SpMat<out_eT> from(const SpBase<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static SpMat<out_eT> from(const SpBase<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static SpMat<out_eT> from(const SpBase<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   
-  template<typename T1>
-  inline static SpMat<out_eT> from(const Base<out_eT, T1>& in);
+  //
+  
+  template<typename in_eT, typename T1>
+  inline static SpMat<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
+  
+  template<typename in_eT, typename T1>
+  inline static SpMat<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   };
 
 
@@ -570,15 +626,177 @@ conv_to< SpMat<out_eT> >::from(const SpBase<in_eT, T1>& in, const typename arma_
 
 
 template<typename out_eT>
-template<typename T1>
+template<typename in_eT, typename T1>
 arma_warn_unused
 inline
 SpMat<out_eT>
-conv_to< SpMat<out_eT> >::from(const Base<out_eT, T1>& in)
+conv_to< SpMat<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk)
   {
   arma_extra_debug_sigprint();
+  arma_ignore(junk);
   
-  return SpMat<out_eT>(in.get_ref());
+  SpMat<out_eT> out;
+  
+  const quasi_unwrap<T1> U(in.get_ref());
+  const Mat<in_eT>&  X = U.M;
+  
+  if(is_same_type<out_eT,in_eT>::yes)
+    {
+    const Mat<out_eT>& Y = reinterpret_cast<const Mat<out_eT>&>(X);
+    
+    SpMat<out_eT> tmp(Y);
+    
+    out.steal_mem(tmp);
+    }
+  else
+    {
+    const uword X_n_rows = X.n_rows;
+    const uword X_n_cols = X.n_cols;
+    const uword X_n_elem = X.n_elem;
+  
+    const in_eT* X_mem = X.memptr();
+    
+    uword X_nnz = 0;
+    
+    for(uword i=0; i < X_n_elem; ++i)  { X_nnz += (X_mem[i] != in_eT(0)) ? uword(1) : uword(0); }
+    
+    podarray< in_eT> X_nonzeros(X_nnz);
+    podarray<out_eT> Y_nonzeros(X_nnz);
+    
+    for(uword i=0,count=0; i < X_n_elem; ++i)
+      {
+      const in_eT X_val = X_mem[i];
+      
+      if(X_val != in_eT(0))  { X_nonzeros[count] = X_val; ++count; }
+      }
+    
+    arrayops::convert( Y_nonzeros.memptr(), X_nonzeros.memptr(), X_nnz );
+    
+    if(X_nnz == 0)
+      {
+      out.set_size(X_n_rows, X.n_cols);
+      }
+    else
+      {
+      SpMat<out_eT> tmp(arma_reserve_indicator(), X_n_rows, X_n_cols, X_nnz);
+      
+      uword count = 0;
+      
+      for(uword c=0; c < X_n_cols; ++c)
+      for(uword r=0; r < X_n_rows; ++r)
+        {
+        const in_eT X_val = (*X_mem);  ++X_mem;
+        
+        if(X_val != in_eT(0))
+          {
+          access::rw(tmp.values[count])      = Y_nonzeros[count];
+          access::rw(tmp.row_indices[count]) = r;
+          access::rw(tmp.col_ptrs[c + 1])++;
+          ++count;
+          }
+        }
+      
+      // Sum column counts to be column pointers.
+      for(uword c=1; c <= tmp.n_cols; ++c)
+        {
+        access::rw(tmp.col_ptrs[c]) += tmp.col_ptrs[c - 1];
+        }
+      
+      tmp.remove_zeros();  // in case conversion resulted in an element equal to zero
+      
+      out.steal_mem(tmp);
+      }
+    }
+  
+  return out;
+  }
+
+
+
+template<typename out_eT>
+template<typename in_eT, typename T1>
+arma_warn_unused
+inline
+SpMat<out_eT>
+conv_to< SpMat<out_eT> >::from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk)
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  SpMat<out_eT> out;
+  
+  const quasi_unwrap<T1> U(in.get_ref());
+  const Mat<in_eT>&  X = U.M;
+  
+  if(is_same_type<out_eT,in_eT>::yes)
+    {
+    const Mat<out_eT>& Y = reinterpret_cast<const Mat<out_eT>&>(X);
+    
+    SpMat<out_eT> tmp(Y);
+    
+    out.steal_mem(tmp);
+    }
+  else
+    {
+    const uword X_n_rows = X.n_rows;
+    const uword X_n_cols = X.n_cols;
+    const uword X_n_elem = X.n_elem;
+  
+    const in_eT* X_mem = X.memptr();
+    
+    uword X_nnz = 0;
+    
+    for(uword i=0; i < X_n_elem; ++i)  { X_nnz += (X_mem[i] != in_eT(0)) ? uword(1) : uword(0); }
+    
+    podarray< in_eT> X_nonzeros(X_nnz);
+    podarray<out_eT> Y_nonzeros(X_nnz);
+    
+    for(uword i=0,count=0; i < X_n_elem; ++i)
+      {
+      const in_eT X_val = X_mem[i];
+      
+      if(X_val != in_eT(0))  { X_nonzeros[count] = X_val; ++count; }
+      }
+    
+    arrayops::convert_cx( Y_nonzeros.memptr(), X_nonzeros.memptr(), X_nnz );
+    
+    if(X_nnz == 0)
+      {
+      out.set_size(X_n_rows, X.n_cols);
+      }
+    else
+      {
+      SpMat<out_eT> tmp(arma_reserve_indicator(), X_n_rows, X_n_cols, X_nnz);
+      
+      uword count = 0;
+      
+      for(uword c=0; c < X_n_cols; ++c)
+      for(uword r=0; r < X_n_rows; ++r)
+        {
+        const in_eT X_val = (*X_mem);  ++X_mem;
+        
+        if(X_val != in_eT(0))
+          {
+          access::rw(tmp.values[count])      = Y_nonzeros[count];
+          access::rw(tmp.row_indices[count]) = r;
+          access::rw(tmp.col_ptrs[c + 1])++;
+          ++count;
+          }
+        }
+      
+      // Sum column counts to be column pointers.
+      for(uword c=1; c <= tmp.n_cols; ++c)
+        {
+        access::rw(tmp.col_ptrs[c]) += tmp.col_ptrs[c - 1];
+        }
+      
+      tmp.remove_zeros();  // in case conversion resulted in an element equal to zero
+      
+      out.steal_mem(tmp);
+      }
+    }
+  
+  return out;
   }
 
 
@@ -590,10 +808,10 @@ class conv_to< Cube<out_eT> >
   public:
   
   template<typename in_eT, typename T1>
-  inline static Cube<out_eT> from(const BaseCube<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static Cube<out_eT> from(const BaseCube<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static Cube<out_eT> from(const BaseCube<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static Cube<out_eT> from(const BaseCube<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   };
 
 
@@ -611,7 +829,7 @@ conv_to< Cube<out_eT> >::from(const BaseCube<in_eT, T1>& in, const typename arma
   const unwrap_cube<T1>  tmp( in.get_ref() );
   const Cube<in_eT>& X = tmp.M;
   
-  Cube<out_eT> out(X.n_rows, X.n_cols, X.n_slices);
+  Cube<out_eT> out(X.n_rows, X.n_cols, X.n_slices, arma_nozeros_indicator());
   
   arrayops::convert( out.memptr(), X.memptr(), X.n_elem );
   
@@ -633,7 +851,7 @@ conv_to< Cube<out_eT> >::from(const BaseCube<in_eT, T1>& in, const typename arma
   const unwrap_cube<T1>  tmp( in.get_ref() );
   const Cube<in_eT>& X = tmp.M;
   
-  Cube<out_eT> out(X.n_rows, X.n_cols, X.n_slices);
+  Cube<out_eT> out(X.n_rows, X.n_cols, X.n_slices, arma_nozeros_indicator());
   
   arrayops::convert_cx( out.memptr(), X.memptr(), X.n_elem );
   
@@ -649,10 +867,10 @@ class conv_to< std::vector<out_eT> >
   public:
   
   template<typename in_eT, typename T1>
-  inline static std::vector<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = 0);
+  inline static std::vector<out_eT> from(const Base<in_eT, T1>& in, const typename arma_not_cx<in_eT>::result* junk = nullptr);
   
   template<typename in_eT, typename T1>
-  inline static std::vector<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = 0);
+  inline static std::vector<out_eT> from(const Base<in_eT, T1>& in, const typename arma_cx_only<in_eT>::result* junk = nullptr);
   };
 
 
@@ -670,7 +888,7 @@ conv_to< std::vector<out_eT> >::from(const Base<in_eT, T1>& in, const typename a
   const quasi_unwrap<T1> tmp(in.get_ref());
   const Mat<in_eT>& X  = tmp.M;
   
-  arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
+  arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object cannot be interpreted as a vector" );
   
   const uword N = X.n_elem;
   
@@ -699,7 +917,7 @@ conv_to< std::vector<out_eT> >::from(const Base<in_eT, T1>& in, const typename a
   const quasi_unwrap<T1> tmp(in.get_ref());
   const Mat<in_eT>& X  = tmp.M;
   
-  arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object can't be interpreted as a vector" );
+  arma_debug_check( ( (X.is_vec() == false) && (X.is_empty() == false) ), "conv_to(): given object cannot be interpreted as a vector" );
   
   const uword N = X.n_elem;
   
