@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -33,11 +35,12 @@ subview_cube_each_common<eT>::subview_cube_each_common(const Cube<eT>& in_p)
 
 
 template<typename eT>
+template<typename eT2>
 inline
 void
-subview_cube_each_common<eT>::check_size(const Mat<eT>& A) const
+subview_cube_each_common<eT>::check_size(const Mat<eT2>& A) const
   {
-  if(arma_config::debug == true)
+  if(arma_config::debug)
     {
     if( (A.n_rows != P.n_rows) || (A.n_cols != P.n_cols) )
       {
@@ -49,12 +52,12 @@ subview_cube_each_common<eT>::check_size(const Mat<eT>& A) const
 
 
 template<typename eT>
-arma_cold
+template<typename eT2>
 inline
 const std::string
-subview_cube_each_common<eT>::incompat_size_string(const Mat<eT>& A) const
+subview_cube_each_common<eT>::incompat_size_string(const Mat<eT2>& A) const
   {
-  std::stringstream tmp;
+  std::ostringstream tmp;
   
   tmp << "each_slice(): incompatible size; expected " << P.n_rows << 'x' << P.n_cols << ", got " << A.n_rows << 'x' << A.n_cols;
   
@@ -132,7 +135,7 @@ subview_cube_each1<eT>::operator+= (const Base<eT,T1>& in)
   const uword p_n_elem_slice = p.n_elem_slice;
   
   const eT* A_mem = A.memptr();
-    
+  
   for(uword i=0; i < p_n_slices; ++i)  { arrayops::inplace_plus( p.slice_memptr(i), A_mem, p_n_elem_slice ); }
   }
 
@@ -285,7 +288,7 @@ subview_cube_each2<eT,TB>::operator= (const Base<eT,T1>& in)
   
   const uword p_n_slices     = p.n_slices;
   const uword p_n_elem_slice = p.n_elem_slice;
-    
+  
   const uword* indices_mem = U.M.memptr();
   const uword  N           = U.M.n_elem;
   
@@ -295,7 +298,7 @@ subview_cube_each2<eT,TB>::operator= (const Base<eT,T1>& in)
     {
     const uword slice = indices_mem[i];
     
-    arma_debug_check( (slice >= p_n_slices), "each_slice(): index out of bounds" );
+    arma_debug_check_bounds( (slice >= p_n_slices), "each_slice(): index out of bounds" );
     
     arrayops::copy(p.slice_memptr(slice), A_mem, p_n_elem_slice);
     }
@@ -324,7 +327,7 @@ subview_cube_each2<eT,TB>::operator+= (const Base<eT,T1>& in)
   
   const uword p_n_slices     = p.n_slices;
   const uword p_n_elem_slice = p.n_elem_slice;
-    
+  
   const uword* indices_mem = U.M.memptr();
   const uword  N           = U.M.n_elem;
   
@@ -334,7 +337,7 @@ subview_cube_each2<eT,TB>::operator+= (const Base<eT,T1>& in)
     {
     const uword slice = indices_mem[i];
     
-    arma_debug_check( (slice >= p_n_slices), "each_slice(): index out of bounds" );
+    arma_debug_check_bounds( (slice >= p_n_slices), "each_slice(): index out of bounds" );
     
     arrayops::inplace_plus(p.slice_memptr(slice), A_mem, p_n_elem_slice);
     }
@@ -363,7 +366,7 @@ subview_cube_each2<eT,TB>::operator-= (const Base<eT,T1>& in)
   
   const uword p_n_slices     = p.n_slices;
   const uword p_n_elem_slice = p.n_elem_slice;
-    
+  
   const uword* indices_mem = U.M.memptr();
   const uword  N           = U.M.n_elem;
   
@@ -373,7 +376,7 @@ subview_cube_each2<eT,TB>::operator-= (const Base<eT,T1>& in)
     {
     const uword slice = indices_mem[i];
     
-    arma_debug_check( (slice >= p_n_slices), "each_slice(): index out of bounds" );
+    arma_debug_check_bounds( (slice >= p_n_slices), "each_slice(): index out of bounds" );
     
     arrayops::inplace_minus(p.slice_memptr(slice), A_mem, p_n_elem_slice);
     }
@@ -402,7 +405,7 @@ subview_cube_each2<eT,TB>::operator%= (const Base<eT,T1>& in)
   
   const uword p_n_slices     = p.n_slices;
   const uword p_n_elem_slice = p.n_elem_slice;
-    
+  
   const uword* indices_mem = U.M.memptr();
   const uword  N           = U.M.n_elem;
   
@@ -412,7 +415,7 @@ subview_cube_each2<eT,TB>::operator%= (const Base<eT,T1>& in)
     {
     const uword slice = indices_mem[i];
     
-    arma_debug_check( (slice >= p_n_slices), "each_slice(): index out of bounds" );
+    arma_debug_check_bounds( (slice >= p_n_slices), "each_slice(): index out of bounds" );
     
     arrayops::inplace_mul(p.slice_memptr(slice), A_mem, p_n_elem_slice);
     }
@@ -441,7 +444,7 @@ subview_cube_each2<eT,TB>::operator/= (const Base<eT,T1>& in)
   
   const uword p_n_slices     = p.n_slices;
   const uword p_n_elem_slice = p.n_elem_slice;
-    
+  
   const uword* indices_mem = U.M.memptr();
   const uword  N           = U.M.n_elem;
   
@@ -451,7 +454,7 @@ subview_cube_each2<eT,TB>::operator/= (const Base<eT,T1>& in)
     {
     const uword slice = indices_mem[i];
     
-    arma_debug_check( (slice >= p_n_slices), "each_slice(): index out of bounds" );
+    arma_debug_check_bounds( (slice >= p_n_slices), "each_slice(): index out of bounds" );
     
     arrayops::inplace_div(p.slice_memptr(slice), A_mem, p_n_elem_slice);
     }
@@ -482,7 +485,7 @@ subview_cube_each1_aux::operator_plus
   const uword p_n_cols   = p.n_cols;
   const uword p_n_slices = p.n_slices;
   
-  Cube<eT> out(p_n_rows, p_n_cols, p_n_slices);
+  Cube<eT> out(p_n_rows, p_n_cols, p_n_slices, arma_nozeros_indicator());
   
   const unwrap<T2>   tmp(Y.get_ref());
   const Mat<eT>& A = tmp.M;
@@ -519,7 +522,7 @@ subview_cube_each1_aux::operator_minus
   const uword p_n_cols   = p.n_cols;
   const uword p_n_slices = p.n_slices;
   
-  Cube<eT> out(p_n_rows, p_n_cols, p_n_slices);
+  Cube<eT> out(p_n_rows, p_n_cols, p_n_slices, arma_nozeros_indicator());
   
   const unwrap<T2>   tmp(Y.get_ref());
   const Mat<eT>& A = tmp.M;
@@ -556,7 +559,7 @@ subview_cube_each1_aux::operator_minus
   const uword p_n_cols   = p.n_cols;
   const uword p_n_slices = p.n_slices;
   
-  Cube<eT> out(p_n_rows, p_n_cols, p_n_slices);
+  Cube<eT> out(p_n_rows, p_n_cols, p_n_slices, arma_nozeros_indicator());
   
   const unwrap<T1>   tmp(X.get_ref());
   const Mat<eT>& A = tmp.M;
@@ -593,7 +596,7 @@ subview_cube_each1_aux::operator_schur
   const uword p_n_cols   = p.n_cols;
   const uword p_n_slices = p.n_slices;
   
-  Cube<eT> out(p_n_rows, p_n_cols, p_n_slices);
+  Cube<eT> out(p_n_rows, p_n_cols, p_n_slices, arma_nozeros_indicator());
   
   const unwrap<T2>   tmp(Y.get_ref());
   const Mat<eT>& A = tmp.M;
@@ -630,7 +633,7 @@ subview_cube_each1_aux::operator_div
   const uword p_n_cols   = p.n_cols;
   const uword p_n_slices = p.n_slices;
   
-  Cube<eT> out(p_n_rows, p_n_cols, p_n_slices);
+  Cube<eT> out(p_n_rows, p_n_cols, p_n_slices, arma_nozeros_indicator());
   
   const unwrap<T2>   tmp(Y.get_ref());
   const Mat<eT>& A = tmp.M;
@@ -667,7 +670,7 @@ subview_cube_each1_aux::operator_div
   const uword p_n_cols   = p.n_cols;
   const uword p_n_slices = p.n_slices;
   
-  Cube<eT> out(p_n_rows, p_n_cols, p_n_slices);
+  Cube<eT> out(p_n_rows, p_n_cols, p_n_slices, arma_nozeros_indicator());
   
   const unwrap<T1>   tmp(X.get_ref());
   const Mat<eT>& A = tmp.M;
@@ -703,7 +706,7 @@ subview_cube_each1_aux::operator_times
   const unwrap<T2>   tmp(Y.get_ref());
   const Mat<eT>& M = tmp.M;
   
-  Cube<eT> out(C.n_rows, M.n_cols, C.n_slices);
+  Cube<eT> out(C.n_rows, M.n_cols, C.n_slices, arma_nozeros_indicator());
   
   for(uword i=0; i < C.n_slices; ++i)
     {
@@ -734,7 +737,7 @@ subview_cube_each1_aux::operator_times
   
   const Cube<eT>& C = Y.P;
   
-  Cube<eT> out(M.n_rows, C.n_cols, C.n_slices);
+  Cube<eT> out(M.n_rows, C.n_cols, C.n_slices, arma_nozeros_indicator());
   
   for(uword i=0; i < C.n_slices; ++i)
     {
@@ -790,7 +793,7 @@ subview_cube_each2_aux::operator_plus
     {
     const uword slice = indices_mem[i];
     
-    arma_debug_check( (slice >= p_n_slices), "each_slice(): index out of bounds" );
+    arma_debug_check_bounds( (slice >= p_n_slices), "each_slice(): index out of bounds" );
     
     arrayops::inplace_plus(out.slice_memptr(slice), A_mem, p_n_elem_slice);
     }
@@ -835,7 +838,7 @@ subview_cube_each2_aux::operator_minus
     {
     const uword slice = indices_mem[i];
     
-    arma_debug_check( (slice >= p_n_slices), "each_slice(): index out of bounds" );
+    arma_debug_check_bounds( (slice >= p_n_slices), "each_slice(): index out of bounds" );
     
     arrayops::inplace_minus(out.slice_memptr(slice), A_mem, p_n_elem_slice);
     }
@@ -879,7 +882,7 @@ subview_cube_each2_aux::operator_minus
     {
     const uword slice = indices_mem[i];
     
-    arma_debug_check( (slice >= p_n_slices), "each_slice(): index out of bounds" );
+    arma_debug_check_bounds( (slice >= p_n_slices), "each_slice(): index out of bounds" );
     
           Mat<eT> out_slice(              out.slice_memptr(slice),  p_n_rows, p_n_cols, false, true);
     const Mat<eT>   p_slice(const_cast<eT*>(p.slice_memptr(slice)), p_n_rows, p_n_cols, false, true);
@@ -927,7 +930,7 @@ subview_cube_each2_aux::operator_schur
     {
     const uword slice = indices_mem[i];
     
-    arma_debug_check( (slice >= p_n_slices), "each_slice(): index out of bounds" );
+    arma_debug_check_bounds( (slice >= p_n_slices), "each_slice(): index out of bounds" );
     
     arrayops::inplace_mul(out.slice_memptr(slice), A_mem, p_n_elem_slice);
     }
@@ -972,7 +975,7 @@ subview_cube_each2_aux::operator_div
     {
     const uword slice = indices_mem[i];
     
-    arma_debug_check( (slice >= p_n_slices), "each_slice(): index out of bounds" );
+    arma_debug_check_bounds( (slice >= p_n_slices), "each_slice(): index out of bounds" );
     
     arrayops::inplace_div(out.slice_memptr(slice), A_mem, p_n_elem_slice);
     }
@@ -1016,7 +1019,7 @@ subview_cube_each2_aux::operator_div
     {
     const uword slice = indices_mem[i];
     
-    arma_debug_check( (slice >= p_n_slices), "each_slice(): index out of bounds" );
+    arma_debug_check_bounds( (slice >= p_n_slices), "each_slice(): index out of bounds" );
     
           Mat<eT> out_slice(              out.slice_memptr(slice),  p_n_rows, p_n_cols, false, true);
     const Mat<eT>   p_slice(const_cast<eT*>(p.slice_memptr(slice)), p_n_rows, p_n_cols, false, true);
